@@ -27,6 +27,7 @@
 #include <libintl.h>
 #include <libhal.h>
 
+#include "apt-worker-client.h"
 #include "dbus.h"
 #include "util.h"
 #include "log.h"
@@ -112,14 +113,14 @@ static void
 dbus_install_packages (DBusConnection *conn, DBusMessage *message)
 {
   DBusError error;
-  
+
   dbus_int32_t xid;
   const char **packages;
   int n_packages;
 
   dbus_connection_ref (conn);
   dbus_message_ref (message);
-  
+
   dip_clos *c = new dip_clos;
   c->conn = conn;
   c->message = message;
@@ -136,7 +137,8 @@ dbus_install_packages (DBusConnection *conn, DBusMessage *message)
     {
       c->xid = xid;
       c->packages = packages;
-      
+
+      maybe_init_packages_list ();
       with_initialized_packages (dip_with_initialized_packages, c);
     }
   else
@@ -192,12 +194,12 @@ dip_end (int result, void *data)
 
   DBusMessage *reply;
   dbus_int32_t dbus_result = result;
-	  
+
   reply = dbus_message_new_method_return (c->message);
   dbus_message_append_args (reply,
 			    DBUS_TYPE_INT32, &dbus_result,
 			    DBUS_TYPE_INVALID);
-  
+
   dbus_connection_send (c->conn, reply, NULL);
   dbus_message_unref (reply);
 
@@ -233,7 +235,7 @@ dbus_install_file (DBusConnection *conn, DBusMessage *message)
 
   dbus_connection_ref (conn);
   dbus_message_ref (message);
-  
+
   dif_clos *c = new dif_clos;
   c->conn = conn;
   c->message = message;
@@ -246,6 +248,8 @@ dbus_install_file (DBusConnection *conn, DBusMessage *message)
 			     DBUS_TYPE_INVALID))
     {
       c->xid = xid;
+
+      maybe_init_packages_list ();
       with_initialized_packages (dif_with_initialized_packages, c);
     }
   else
@@ -299,12 +303,12 @@ dif_end (int result, void *data)
 
   DBusMessage *reply;
   dbus_int32_t dbus_result = result;
-	  
+
   reply = dbus_message_new_method_return (c->message);
   dbus_message_append_args (reply,
 			    DBUS_TYPE_INT32, &dbus_result,
 			    DBUS_TYPE_INVALID);
-  
+
   dbus_connection_send (c->conn, reply, NULL);
   dbus_message_unref (reply);
 
@@ -318,6 +322,55 @@ dif_end (int result, void *data)
   maybe_exit ();
 }
 
+static void
+dbus_top_application (DBusConnection *conn, DBusMessage *message)
+{
+  DBusMessage *reply;
+
+  present_main_window ();
+  maybe_init_packages_list ();
+
+  reply = dbus_message_new_method_return (message);
+  dbus_connection_send (conn, reply, NULL);
+  dbus_message_unref (reply);
+}
+
+static void
+dbus_show_check_for_updates_view (DBusConnection *conn, DBusMessage *message)
+{
+  DBusMessage *reply;
+
+  present_main_window ();
+  maybe_init_packages_list ();
+
+  if (is_idle ())
+    show_check_for_updates_view ();
+
+  reply = dbus_message_new_method_return (message);
+  dbus_connection_send (conn, reply, NULL);
+  dbus_message_unref (reply);
+}
+
+static void
+dbus_showing_check_for_updates_view (DBusConnection *conn, DBusMessage *message)
+{
+  DBusMessage *reply;
+  gboolean showing_view = FALSE;
+
+  /* Check if 'check for updates' view is being shown */
+  if (get_current_view_id () == UPGRADE_APPLICATIONS_VIEW)
+    showing_view = TRUE;
+
+  /* Build reply message with the required boolean value */
+  reply = dbus_message_new_method_return (message);
+  dbus_message_append_args (reply,
+                            DBUS_TYPE_BOOLEAN , &showing_view,
+                            DBUS_TYPE_INVALID);
+
+  dbus_connection_send (conn, reply, NULL);
+  dbus_message_unref (reply);
+}
+
 static void icfu_end (bool ignored, void *data);
 
 static void
@@ -332,7 +385,86 @@ icfu_end (bool ignored, void *data)
   end_interaction_flow ();
 }
 
-static DBusHandlerResult 
+static void
+dbus_check_for_updates (DBusConnection *conn, DBusMessage *message)
+{
+  DBusMessage *reply;
+
+  maybe_init_packages_list ();
+  start_interaction_flow_when_idle (idle_check_for_updates, NULL);
+
+  reply = dbus_message_new_method_return (message);
+  dbus_connection_send (conn, reply, NULL);
+  dbus_message_unref (reply);
+}
+
+static void dsp_with_initialized_packages (void *data);
+
+static void
+dbus_search_packages (DBusConnection *conn, DBusMessage *message)
+{
+  DBusError error;
+  const gchar *arg_pattern;
+  gchar *pattern;
+  dbus_int32_t dbus_result = 1;
+
+  dbus_connection_ref (conn);
+  dbus_message_ref (message);
+
+  dbus_error_init (&error);
+  if (dbus_message_get_args (message, &error,
+			     DBUS_TYPE_STRING, &arg_pattern,
+			     DBUS_TYPE_INVALID))
+    {
+      pattern = g_strdup (arg_pattern);
+
+      if (pattern != NULL)
+        {
+          maybe_init_packages_list ();
+          with_initialized_packages (dsp_with_initialized_packages, pattern);
+        }
+    }
+  else
+    {
+      DBusMessage *reply;
+      reply = dbus_message_new_error (message,
+				      DBUS_ERROR_INVALID_ARGS,
+				      error.message);
+      dbus_connection_send (conn, reply, NULL);
+      dbus_message_unref (reply);
+      dbus_result = -1;
+    }
+
+  DBusMessage * reply = dbus_message_new_method_return (message);
+  dbus_message_append_args (reply,
+                            DBUS_TYPE_INT32, &dbus_result,
+                            DBUS_TYPE_INVALID);
+
+  dbus_connection_send (conn, reply, NULL);
+  dbus_message_unref (reply);
+
+  // So that we don't lose the reply when we exit below.
+  dbus_connection_flush (conn);
+
+  dbus_message_unref (message);
+  dbus_connection_unref (conn);
+}
+
+static void
+dsp_with_initialized_packages (void *data)
+{
+  gchar *pattern = (gchar *) data;
+
+  present_main_window ();
+  if (is_idle ())
+    {
+      show_install_applications_view ();
+      search_packages (pattern, true);
+    }
+  g_free (pattern);
+}
+
+static DBusHandlerResult
 dbus_handler (DBusConnection *conn, DBusMessage *message, void *data)
 {
   if (dbus_message_is_method_call (message,
@@ -363,14 +495,7 @@ dbus_handler (DBusConnection *conn, DBusMessage *message, void *data)
 				   "com.nokia.hildon_application_manager",
 				   "top_application"))
     {
-      DBusMessage *reply;
-
-      present_main_window ();
-
-      reply = dbus_message_new_method_return (message);
-      dbus_connection_send (conn, reply, NULL);
-      dbus_message_unref (reply);
-
+      dbus_top_application (conn, message);
       return DBUS_HANDLER_RESULT_HANDLED;
     }
 
@@ -378,16 +503,7 @@ dbus_handler (DBusConnection *conn, DBusMessage *message, void *data)
 				   "com.nokia.hildon_application_manager",
 				   "show_check_for_updates_view"))
     {
-      DBusMessage *reply;
-
-      present_main_window ();
-      if (is_idle ())
-	show_check_for_updates_view ();
-      
-      reply = dbus_message_new_method_return (message);
-      dbus_connection_send (conn, reply, NULL);
-      dbus_message_unref (reply);
-
+      dbus_show_check_for_updates_view (conn, message);
       return DBUS_HANDLER_RESULT_HANDLED;
     }
 
@@ -395,22 +511,7 @@ dbus_handler (DBusConnection *conn, DBusMessage *message, void *data)
 				   "com.nokia.hildon_application_manager",
 				   "showing_check_for_updates_view"))
     {
-      DBusMessage *reply;
-      gboolean showing_view = FALSE;
-
-      /* Check if 'check for updates' view is being shown */
-      if (get_current_view_id () == UPGRADE_APPLICATIONS_VIEW)
-	showing_view = TRUE;
-
-      /* Build reply message with the required boolean value */
-      reply = dbus_message_new_method_return (message);
-      dbus_message_append_args (reply,
-				DBUS_TYPE_BOOLEAN , &showing_view,
-				DBUS_TYPE_INVALID);
-
-      dbus_connection_send (conn, reply, NULL);
-      dbus_message_unref (reply);
-
+      dbus_showing_check_for_updates_view (conn, message);
       return DBUS_HANDLER_RESULT_HANDLED;
     }
 
@@ -418,14 +519,15 @@ dbus_handler (DBusConnection *conn, DBusMessage *message, void *data)
 				   "com.nokia.hildon_application_manager",
 				   "check_for_updates"))
     {
-      DBusMessage *reply;
-
-      start_interaction_flow_when_idle (idle_check_for_updates, NULL);
-      
-      reply = dbus_message_new_method_return (message);
-      dbus_connection_send (conn, reply, NULL);
-      dbus_message_unref (reply);
+      dbus_check_for_updates (conn, message);
+      return DBUS_HANDLER_RESULT_HANDLED;
+    }
 
+  if (dbus_message_is_method_call (message,
+                                   "com.nokia.hildon_application_manager",
+                                   "search_packages"))
+    {
+      dbus_search_packages (conn, message);
       return DBUS_HANDLER_RESULT_HANDLED;
     }
 
