@@ -267,6 +267,7 @@ static void ip_not_enough_battery_confirm (void (*cont) (void *data), void *data
 static void ip_not_enough_battery_confirm_response (void *data);
 
 static void ip_install_one (void *data);
+static void ip_install_one_with_space_checked (int cmd, apt_proto_decoder *dec, void *data);
 static void ip_maybe_continue (bool res, void *data);
 
 static void ip_execute_checkrm_script (const char *name,
@@ -285,8 +286,9 @@ static void ip_download_cur_reply (int cmd, apt_proto_decoder *dec, void *data);
 static void ip_download_cur_fail (void *data);
 static void ip_download_cur_retry_confirm (apt_proto_result_code result_code, void *data);
 static void ip_download_cur_retry_confirm_response (bool result, void *data);
-static gboolean ip_stop_hsm_and_install_delayed (gpointer data);
+static gboolean ip_kill_all_and_install_delayed (gpointer data);
 static void ip_install_cur (void *data);
+static void ip_install_cur_with_space_checked (int cmd, apt_proto_decoder *dec, void *data);
 static void ip_install_cur_reply (int cmd, apt_proto_decoder *dec, void *data);
 static void ip_clean_reply (int cmd, apt_proto_decoder *dec, void *data);
 static void ip_install_next (void *data);
@@ -467,7 +469,7 @@ ip_install_with_info (void *data)
 	      break;
 #endif
 	    default:
-	      desc = _("ai_tb_install");
+	      desc = _("ai_li_install");
 	      break;
 	    }
 	}
@@ -935,7 +937,8 @@ ip_warn_about_reboot_response (GtkDialog *dialog, gint response,
 	{
 	  package_info *pi = (package_info *)(c->cur->data);
 
-	  close_apps ();
+          close_apps ();
+          set_prestarted_apps_enabled (FALSE);
 
 	  /* Convert the entertainment dialog into system modal if the
 	     package to be installed requires rebooting the system */
@@ -1070,7 +1073,23 @@ ip_install_one (void *data)
       return;
     }
 
-  int64_t free_space = get_free_space ();
+  /* Check free space to install */
+  apt_worker_get_free_space (ip_install_one_with_space_checked, c);
+}
+
+static void
+ip_install_one_with_space_checked (int cmd, apt_proto_decoder *dec, void *data)
+{
+  ip_clos *c = (ip_clos *)data;
+  package_info *pi = (package_info *)(c->cur->data);
+
+  if (dec == NULL)
+    {
+      ip_end (c);
+      return;
+    }
+
+  int64_t free_space = dec->decode_int64 ();
 
   if (free_space < 0)
     annoy_user_with_errno (errno, "get_free_space",
@@ -1415,7 +1434,7 @@ ip_download_cur_reply (int cmd, apt_proto_decoder *dec, void *data)
 }
 
 static gboolean
-ip_stop_hsm_and_install_delayed (gpointer data)
+ip_kill_all_and_install_delayed (gpointer data)
 {
   ip_clos *c = (ip_clos *)data;
 
@@ -1423,8 +1442,30 @@ ip_stop_hsm_and_install_delayed (gpointer data)
     {
       package_info *pi = (package_info *)(c->cur->data);
 
-      /* Stop hildon-status-menu and continue with installation */
+      /* Kill running processes that could interfere for SSU */
+      /* Disclaimer: this is a nasty workaround to prevent from a
+         bigger problem to happen because of unknown reasons atm.
+         Programmer avoids any responsibility on this code,
+         implemented under high pressure as requested "from above". */
+      maybe_kill_all_by_name ("rtcom-messaging-ui", SIGHUP);
+      run_cmd_simple ("/etc/init.d/alarmd stop");
+      stop_dsme_service ("/usr/bin/camera-ui");
+      stop_dsme_service ("/usr/sbin/browserd -d");
       stop_dsme_service ("/usr/bin/hildon-status-menu");
+      stop_dsme_service ("/usr/bin/hildon-home");
+      stop_dsme_service ("/usr/bin/hildon-input-method");
+      stop_dsme_service ("/usr/bin/intellisyncd");
+      stop_dsme_service ("/usr/bin/clipboard-manager");
+      stop_dsme_service ("/usr/bin/syncd");
+      stop_dsme_service ("/usr/bin/hildon-desktop");
+      stop_dsme_service ("/usr/bin/osso-connectivity-ui-conndlgs");
+      maybe_kill_all_by_name ("syncd", SIGKILL);
+      maybe_kill_all_by_name ("trackerd", SIGKILL);
+      maybe_kill_all_by_name ("intellisyncd", SIGKILL);
+      maybe_kill_all_by_name ("osso-abook-home-applet", SIGKILL);
+      maybe_kill_all_by_name ("hildon-thumbnailerd", SIGKILL);
+      maybe_kill_all_by_name ("maemo-xinput-sounds", SIGKILL);
+
       apt_worker_install_package (pi->name,
                                   c->alt_download_root,
                                   ip_install_cur_reply, c);
@@ -1453,7 +1494,22 @@ ip_install_cur (void *data)
     }
 
   /* Check free space before downloading */
-  int64_t free_space = get_free_space ();
+  apt_worker_get_free_space (ip_install_cur_with_space_checked, c);
+}
+
+static void
+ip_install_cur_with_space_checked (int cmd, apt_proto_decoder *dec, void *data)
+{
+  ip_clos *c = (ip_clos *)data;
+  package_info *pi = (package_info *)(c->cur->data);
+
+  if (dec == NULL)
+    {
+      ip_end (c);
+      return;
+    }
+
+  int64_t free_space = dec->decode_int64 ();
   if (free_space < 0)
     annoy_user_with_errno (errno, "get_free_space",
 			   ip_end, c);
@@ -1469,7 +1525,7 @@ ip_install_cur (void *data)
            * problem happening sometimes when installing an SSU, but
            * wait some time first because of the offline mode change
            * (and 3 seconds should be more than enough) */
-          g_timeout_add (3000, ip_stop_hsm_and_install_delayed, c);
+          g_timeout_add (3000, ip_kill_all_and_install_delayed, c);
         }
       else
         {
@@ -1493,6 +1549,8 @@ ip_install_cur_reply (int cmd, apt_proto_decoder *dec, void *data)
   ip_clos *c = (ip_clos *)data;
   package_info *pi = (package_info *)(c->cur->data);
 
+  bool needs_reboot = package_needs_reboot (pi);
+
   if (dec == NULL)
     {
       ip_end (c);
@@ -1502,22 +1560,26 @@ ip_install_cur_reply (int cmd, apt_proto_decoder *dec, void *data)
   apt_proto_result_code result_code =
     apt_proto_result_code (dec->decode_int ());
 
-  if (clean_after_install)
-    apt_worker_clean (ip_clean_reply, NULL);
+  if (clean_after_install
+      && ((result_code == rescode_success) || !needs_reboot))
+    {
+      /* Clean only when needed */
+      apt_worker_clean (ip_clean_reply, NULL);
+    }
 
   c->refresh_needed = true;
 
+  /* Save the backup data right after installing the package */
   if (result_code == rescode_success)
+    save_backup_data ();
+
+  /* Reboot if needed */
+  if (needs_reboot)
+    ip_reboot (c);
+  else if (result_code == rescode_success)
     {
       c->n_successful += 1;
-
-      /* Save the backup data right after installing the package */
-      save_backup_data ();
-
-      if (package_needs_reboot (pi))
-	ip_reboot (c);
-      else
-	ip_install_next (c);
+      ip_install_next (c);
     }
   else
     {
@@ -1766,6 +1828,9 @@ ip_end (void *data)
   /* restore the device mode if needed */
   ip_maybe_restore_device_mode (c);
 
+  /* Make sure prestarted apps are enabled */
+  set_prestarted_apps_enabled (TRUE);
+
   if (c->entertaining)
     stop_entertaining_user ();
 
